==> download.php?FileName=terralib-5.3.2.zip: src/terralib/qt/widgets/utils/DoubleListWidget.cpp <==
/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/utils/DoubleListWidget.cpp

  \brief ...
*/

// TerraLib
#include "ui_DoubleListWidgetForm.h"
#include "DoubleListWidget.h"

// Qt
#include <QIcon>


te::qt::widgets::DoubleListWidget::DoubleListWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
    m_ui(new Ui::DoubleListWidgetForm)
{
  m_ui->setupUi(this);

// set icons
  m_ui->m_addToolButton->setIcon(QIcon::fromTheme("go-next"));
  m_ui->m_addAllToolButton->setIcon(QIcon::fromTheme("go-last"));
  m_ui->m_removeToolButton->setIcon(QIcon::fromTheme("go-previous"));
  m_ui->m_removeAllToolButton->setIcon(QIcon::fromTheme("go-first"));
  m_ui->m_upToolButton->setIcon(QIcon::fromTheme("go-up"));
  m_ui->m_downToolButton->setIcon(QIcon::fromTheme("go-down"));

// set selection mode
  m_ui->m_leftListWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
  m_ui->m_rightListWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);

// connect signals and slots
  connect(m_ui->m_addToolButton, SIGNAL(pressed()), this, SLOT(onAddToolButtonPressed()));
  connect(m_ui->m_addAllToolButton, SIGNAL(pressed()), this, SLOT(onAddAllToolButtonPressed()));
  connect(m_ui->m_removeToolButton, SIGNAL(pressed()), this, SLOT(onRemoveToolButtonPressed()));
  connect(m_ui->m_removeAllToolButton, SIGNAL(pressed()), this, SLOT(onRemoveAllToolButtonPressed()));
  connect(m_ui->m_leftListWidget, SIGNAL(itemSelectionChanged()), this, SLOT(onLeftListItemSelectionChanged()));
  connect(m_ui->m_rightListWidget, SIGNAL(itemSelectionChanged()), this, SLOT(onRightListItemSelectionChanged()));
  connect(m_ui->m_upToolButton, SIGNAL(pressed()), this, SLOT(onUpToolButtonPressed()));
  connect(m_ui->m_downToolButton, SIGNAL(pressed()), this, SLOT(onDownToolButtonPressed()));
}

te::qt::widgets::DoubleListWidget::~DoubleListWidget()
{
}

Ui::DoubleListWidgetForm* te::qt::widgets::DoubleListWidget::getForm() const
{
  return m_ui.get();
}

void te::qt::widgets::DoubleListWidget::setInputValues(std::vector<std::string> values)
{
  m_ui->m_leftListWidget->clear();

  if(values.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);
  else
    m_ui->m_addAllToolButton->setEnabled(false);

  for(size_t i = 0; i < values.size(); ++i)
  {
    m_ui->m_leftListWidget->addItem(QString::fromUtf8(values[i].c_str()));
  }
}

void te::qt::widgets::DoubleListWidget::setInputDataValues(std::vector<std::string> values, std::vector<int> ids)
{
  m_ui->m_leftListWidget->clear();

  if (values.size() != ids.size())
    return;

  if (values.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);
  else
    m_ui->m_addAllToolButton->setEnabled(false);

  for (size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_leftListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setData(Qt::UserRole, QVariant(ids[i]));

    m_ui->m_leftListWidget->addItem(item);
  }
}

void te::qt::widgets::DoubleListWidget::setOutputValues(std::vector<std::string> values)
{
  m_ui->m_rightListWidget->clear();

  if(values.empty() == false)
    m_ui->m_removeAllToolButton->setEnabled(true);
  else
    m_ui->m_removeAllToolButton->setEnabled(false);

  for(size_t i = 0; i < values.size(); ++i)
  {
    m_ui->m_rightListWidget->addItem(QString::fromUtf8(values[i].c_str()));
  }
}

void te::qt::widgets::DoubleListWidget::setOutputDataValues(std::vector<std::string> values, std::vector<int> ids)
{
  m_ui->m_rightListWidget->clear();

  if (values.size() != ids.size())
    return;

  if (values.empty() == false)
    m_ui->m_removeAllToolButton->setEnabled(true);
  else
    m_ui->m_removeAllToolButton->setEnabled(false);

  for (size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_rightListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setData(Qt::UserRole, QVariant(ids[i]));

    m_ui->m_rightListWidget->addItem(item);
  }
}

void te::qt::widgets::DoubleListWidget::setFixedOutputValues(std::vector<std::string> values, std::string iconName)
{
  for(size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_rightListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setFlags(Qt::ItemIsEnabled);
    item->setIcon(QIcon::fromTheme(iconName.c_str()));
  }
}

void te::qt::widgets::DoubleListWidget::setLeftLabel(const QString& value)
{
  m_ui->m_leftItemsLabel->setText(value);
}

void te::qt::widgets::DoubleListWidget::setRightLabel(const QString& value)
{
  m_ui->m_rightItemsLabel->setText(value);
}

std::vector<std::string> te::qt::widgets::DoubleListWidget::getOutputValues()
{
  std::vector<std::string> vec;

  int count = m_ui->m_rightListWidget->count();

  for(int i = 0; i < count; ++i)
  {
    vec.push_back(m_ui->m_rightListWidget->item(i)->text().toUtf8().data());
  }

  return vec;
}

std::vector<int> te::qt::widgets::DoubleListWidget::getOutputDataValues()
{
  std::vector<int> vec;

  int count = m_ui->m_rightListWidget->count();

  for (int i = 0; i < count; ++i)
  {
    QVariant v = m_ui->m_rightListWidget->item(i)->data(Qt::UserRole);

    if (v.isValid())
      vec.push_back(v.toInt());
  }

  return vec;
}

void te::qt::widgets::DoubleListWidget::clearInputValues()
{
  m_ui->m_leftListWidget->clear();

  m_ui->m_addToolButton->setEnabled(false);
  m_ui->m_addAllToolButton->setEnabled(false);
}

void te::qt::widgets::DoubleListWidget::clearOutputValues()
{
  m_ui->m_rightListWidget->clear();

  m_ui->m_removeToolButton->setEnabled(false);
  m_ui->m_removeAllToolButton->setEnabled(false);
}

void te::qt::widgets::DoubleListWidget::setVisibledOrderButtons(bool state)
{
  m_ui->m_upToolButton->setVisible(state);
  m_ui->m_downToolButton->setVisible(state);
}

void te::qt::widgets::DoubleListWidget::addItems(QList<QListWidgetItem*> list)
{
  std::vector<int> itemList;

  for (int i = 0; i < list.size(); ++i)
  {
    int row = m_ui->m_leftListWidget->row(list[i]);

    QListWidgetItem* item = m_ui->m_leftListWidget->takeItem(row);

    m_ui->m_rightListWidget->addItem(item);

    QVariant v = item->data(Qt::UserRole);

    if (v.isValid())
      itemList.push_back(v.toInt());
  }

  m_ui->m_addToolButton->setEnabled(false);
  m_ui->m_addAllToolButton->setEnabled(m_ui->m_leftListWidget->count() != 0);

  emit itemAdded(itemList);
  emit itemChanged();
}

void te::qt::widgets::DoubleListWidget::addItems(std::vector<int> ids)
{
  QList<QListWidgetItem*> listItem;

  for (std::size_t i = 0; i < ids.size(); ++i)
  {
    QListWidgetItem* item = getItemFromId(ids[(int)i]);

    if (item != nullptr)
      listItem.push_back(item);
  }

  if (!listItem.isEmpty())
    addItems(listItem);
}

void te::qt::widgets::DoubleListWidget::removeItems(QList<QListWidgetItem*> list)
{
  std::vector<int> itemList;

  for (int i = 0; i < list.size(); ++i)
  {
    int row = m_ui->m_rightListWidget->row(list[i]);

    QListWidgetItem* item = m_ui->m_rightListWidget->takeItem(row);

    m_ui->m_leftListWidget->addItem(item);

    QVariant v = item->data(Qt::UserRole);

    if (v.isValid())
      itemList.push_back(v.toInt());
  }
  
  m_ui->m_removeToolButton->setEnabled(false);
  m_ui->m_removeAllToolButton->setEnabled(m_ui->m_rightListWidget->count() != 0);
  m_ui->m_upToolButton->setEnabled(m_ui->m_rightListWidget->count() != 0);
  m_ui->m_downToolButton->setEnabled(m_ui->m_rightListWidget->count() != 0);

  emit itemRemoved(itemList);
  emit itemChanged();
}

void te::qt::widgets::DoubleListWidget::removeItems(std::vector<int> ids)
{
  QList<QListWidgetItem*> listItem;

  for (std::size_t i = 0; i < ids.size(); ++i)
  {
    QListWidgetItem* item = getItemFromId(ids[(int)i]);

    if (item != nullptr)
      listItem.push_back(item);
  }

  if (!listItem.isEmpty())
    removeItems(listItem);
}

void te::qt::widgets::DoubleListWidget::onAddToolButtonPressed()
{
  QList<QListWidgetItem*> list = m_ui->m_leftListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_removeAllToolButton->setEnabled(true);

  addItems(list);
}

void te::qt::widgets::DoubleListWidget::onAddAllToolButtonPressed()
{
  m_ui->m_leftListWidget->selectAll();

  QList<QListWidgetItem*> list = m_ui->m_leftListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_removeAllToolButton->setEnabled(true);

  addItems(list);
}

void te::qt::widgets::DoubleListWidget::onRemoveToolButtonPressed()
{
  QList<QListWidgetItem*> list = m_ui->m_rightListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);

  removeItems(list);
}

void te::qt::widgets::DoubleListWidget::onRemoveAllToolButtonPressed()
{
  m_ui->m_rightListWidget->selectAll();

  QList<QListWidgetItem*> list = m_ui->m_rightListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);

  removeItems(list);
}

void te::qt::widgets::DoubleListWidget::onDownToolButtonPressed()
{
  int count = m_ui->m_rightListWidget->count();

  if(count == m_ui->m_rightListWidget->selectedItems().size())
    return;

  for(int i = count-1; i >= 0; --i)
  {
    QListWidgetItem* currentItem = m_ui->m_rightListWidget->item(i);

    if(i == count-1)
      continue;

    if(currentItem->isSelected())
    {
      QListWidgetItem* itemAfter = m_ui->m_rightListWidget->item(i+1);

      bool wasSelected = itemAfter->isSelected();

      m_ui->m_rightListWidget->takeItem(i);
      m_ui->m_rightListWidget->takeItem(i);

      m_ui->m_rightListWidget->insertItem(i, itemAfter);
      m_ui->m_rightListWidget->insertItem(i+1, currentItem);

      currentItem->setSelected(true);

      if(wasSelected)
        itemAfter->setSelected(true);
    }
  }
}

void te::qt::widgets::DoubleListWidget::onUpToolButtonPressed()
{
  int count = m_ui->m_rightListWidget->count();

  if(count == m_ui->m_rightListWidget->selectedItems().size())
    return;

  for(int i = 0; i < count; ++i)
  {
    QListWidgetItem* currentItem = m_ui->m_rightListWidget->item(i);

    if(i == 0)
      continue;

    if(currentItem->isSelected())
    {
      QListWidgetItem* itemBefore = m_ui->m_rightListWidget->item(i-1);

      bool wasSelected = itemBefore->isSelected();

      m_ui->m_rightListWidget->takeItem(i);
      m_ui->m_rightListWidget->takeItem(i-1);

      m_ui->m_rightListWidget->insertItem(i-1, currentItem);
      m_ui->m_rightListWidget->insertItem(i, itemBefore);

      currentItem->setSelected(true);

      if(wasSelected)
        itemBefore->setSelected(true);
    }
  }
}

void te::qt::widgets::DoubleListWidget::onLeftListItemSelectionChanged()
{
  if(m_ui->m_leftListWidget->selectedItems().size() > 0)
  {
    m_ui->m_addToolButton->setEnabled(true);
    m_ui->m_addAllToolButton->setEnabled(true);
  }
  else
  {
    m_ui->m_addToolButton->setEnabled(false);
  }
}

void te::qt::widgets::DoubleListWidget::onRightListItemSelectionChanged()
{
  if(m_ui->m_rightListWidget->selectedItems().size() > 0)
  {
    m_ui->m_removeToolButton->setEnabled(true);
    m_ui->m_removeAllToolButton->setEnabled(true);
    m_ui->m_upToolButton->setEnabled(true);
    m_ui->m_downToolButton->setEnabled(true);
  }
  else
  {
    m_ui->m_removeToolButton->setEnabled(false);
    m_ui->m_upToolButton->setEnabled(false);
    m_ui->m_downToolButton->setEnabled(false);
  }
}



QListWidgetItem* te::qt::widgets::DoubleListWidget::getItemFromId(const int &id)
{
  int count = m_ui->m_leftListWidget->count();

  for (int i = 0; i < count; ++i)
  {
    QVariant v = m_ui->m_leftListWidget->item(i)->data(Qt::UserRole);
    if (!v.isValid())
      continue;

    if (v.toInt() == id)
      return m_ui->m_leftListWidget->item(i);
  }

  count = m_ui->m_rightListWidget->count();

  for (int i = 0; i < count; ++i)
  {
    QVariant v = m_ui->m_rightListWidget->item(i)->data(Qt::UserRole);
    if (!v.isValid())
      continue;

    if (v.toInt() == id)
      return m_ui->m_rightListWidget->item(i);
  }

  return nullptr;
}

==> download.php?FileName=terralib-5.4.0.zip: src/terralib/qt/widgets/utils/DoubleListWidget.cpp <==
/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/utils/DoubleListWidget.cpp

  \brief ...
*/

// TerraLib
#include "ui/ui_DoubleListWidgetForm.h"
#include "DoubleListWidget.h"

// Qt
#include <QIcon>


te::qt::widgets::DoubleListWidget::DoubleListWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
    m_ui(new Ui::DoubleListWidgetForm)
{
  m_ui->setupUi(this);

// set icons
  m_ui->m_addToolButton->setIcon(QIcon::fromTheme("go-next"));
  m_ui->m_addAllToolButton->setIcon(QIcon::fromTheme("go-last"));
  m_ui->m_removeToolButton->setIcon(QIcon::fromTheme("go-previous"));
  m_ui->m_removeAllToolButton->setIcon(QIcon::fromTheme("go-first"));
  m_ui->m_upToolButton->setIcon(QIcon::fromTheme("go-up"));
  m_ui->m_downToolButton->setIcon(QIcon::fromTheme("go-down"));

// set selection mode
  m_ui->m_leftListWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
  m_ui->m_rightListWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);

// connect signals and slots
  connect(m_ui->m_addToolButton, SIGNAL(pressed()), this, SLOT(onAddToolButtonPressed()));
  connect(m_ui->m_addAllToolButton, SIGNAL(pressed()), this, SLOT(onAddAllToolButtonPressed()));
  connect(m_ui->m_removeToolButton, SIGNAL(pressed()), this, SLOT(onRemoveToolButtonPressed()));
  connect(m_ui->m_removeAllToolButton, SIGNAL(pressed()), this, SLOT(onRemoveAllToolButtonPressed()));
  connect(m_ui->m_leftListWidget, SIGNAL(itemSelectionChanged()), this, SLOT(onLeftListItemSelectionChanged()));
  connect(m_ui->m_rightListWidget, SIGNAL(itemSelectionChanged()), this, SLOT(onRightListItemSelectionChanged()));
  connect(m_ui->m_upToolButton, SIGNAL(pressed()), this, SLOT(onUpToolButtonPressed()));
  connect(m_ui->m_downToolButton, SIGNAL(pressed()), this, SLOT(onDownToolButtonPressed()));
}

te::qt::widgets::DoubleListWidget::~DoubleListWidget() = default;

Ui::DoubleListWidgetForm* te::qt::widgets::DoubleListWidget::getForm() const
{
  return m_ui.get();
}

void te::qt::widgets::DoubleListWidget::setInputValues(std::vector<std::string> values)
{
  m_ui->m_leftListWidget->clear();

  if(values.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);
  else
    m_ui->m_addAllToolButton->setEnabled(false);

  for(size_t i = 0; i < values.size(); ++i)
  {
    m_ui->m_leftListWidget->addItem(QString::fromUtf8(values[i].c_str()));
  }
}

void te::qt::widgets::DoubleListWidget::setInputDataValues(std::vector<std::string> values, std::vector<int> ids)
{
  m_ui->m_leftListWidget->clear();

  if (values.size() != ids.size())
    return;

  if (values.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);
  else
    m_ui->m_addAllToolButton->setEnabled(false);

  for (size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_leftListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setData(Qt::UserRole, QVariant(ids[i]));

    m_ui->m_leftListWidget->addItem(item);
  }
}

void te::qt::widgets::DoubleListWidget::setOutputValues(std::vector<std::string> values)
{
  m_ui->m_rightListWidget->clear();

  if(values.empty() == false)
    m_ui->m_removeAllToolButton->setEnabled(true);
  else
    m_ui->m_removeAllToolButton->setEnabled(false);

  for(size_t i = 0; i < values.size(); ++i)
  {
    m_ui->m_rightListWidget->addItem(QString::fromUtf8(values[i].c_str()));
  }
}

void te::qt::widgets::DoubleListWidget::setOutputDataValues(std::vector<std::string> values, std::vector<int> ids)
{
  m_ui->m_rightListWidget->clear();

  if (values.size() != ids.size())
    return;

  if (values.empty() == false)
    m_ui->m_removeAllToolButton->setEnabled(true);
  else
    m_ui->m_removeAllToolButton->setEnabled(false);

  for (size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_rightListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setData(Qt::UserRole, QVariant(ids[i]));

    m_ui->m_rightListWidget->addItem(item);
  }
}

void te::qt::widgets::DoubleListWidget::setFixedOutputValues(std::vector<std::string> values, std::string iconName)
{
  for(size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_rightListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setFlags(Qt::ItemIsEnabled);
    item->setIcon(QIcon::fromTheme(iconName.c_str()));
  }
}

void te::qt::widgets::DoubleListWidget::setLeftLabel(const QString& value)
{
  m_ui->m_leftItemsLabel->setText(value);
}

void te::qt::widgets::DoubleListWidget::setRightLabel(const QString& value)
{
  m_ui->m_rightItemsLabel->setText(value);
}

std::vector<std::string> te::qt::widgets::DoubleListWidget::getOutputValues()
{
  std::vector<std::string> vec;

  int count = m_ui->m_rightListWidget->count();

  for(int i = 0; i < count; ++i)
  {
    vec.push_back(m_ui->m_rightListWidget->item(i)->text().toUtf8().data());
  }

  return vec;
}

std::vector<int> te::qt::widgets::DoubleListWidget::getOutputDataValues()
{
  std::vector<int> vec;

  int count = m_ui->m_rightListWidget->count();

  for (int i = 0; i < count; ++i)
  {
    QVariant v = m_ui->m_rightListWidget->item(i)->data(Qt::UserRole);

    if (v.isValid())
      vec.push_back(v.toInt());
  }

  return vec;
}

void te::qt::widgets::DoubleListWidget::clearInputValues()
{
  m_ui->m_leftListWidget->clear();

  m_ui->m_addToolButton->setEnabled(false);
  m_ui->m_addAllToolButton->setEnabled(false);
}

void te::qt::widgets::DoubleListWidget::clearOutputValues()
{
  m_ui->m_rightListWidget->clear();

  m_ui->m_removeToolButton->setEnabled(false);
  m_ui->m_removeAllToolButton->setEnabled(false);
}

void te::qt::widgets::DoubleListWidget::setVisibledOrderButtons(bool state)
{
  m_ui->m_upToolButton->setVisible(state);
  m_ui->m_downToolButton->setVisible(state);
}

void te::qt::widgets::DoubleListWidget::addItems(QList<QListWidgetItem*> list)
{
  std::vector<int> itemList;

  for (int i = 0; i < list.size(); ++i)
  {
    int row = m_ui->m_leftListWidget->row(list[i]);

    QListWidgetItem* item = m_ui->m_leftListWidget->takeItem(row);

    m_ui->m_rightListWidget->addItem(item);

    QVariant v = item->data(Qt::UserRole);

    if (v.isValid())
      itemList.push_back(v.toInt());
  }

  m_ui->m_addToolButton->setEnabled(false);
  m_ui->m_addAllToolButton->setEnabled(m_ui->m_leftListWidget->count() != 0);

  emit itemAdded(itemList);
  emit itemChanged();
}

void te::qt::widgets::DoubleListWidget::addItems(std::vector<int> ids)
{
  QList<QListWidgetItem*> listItem;

  for (std::size_t i = 0; i < ids.size(); ++i)
  {
    QListWidgetItem* item = getItemFromId(ids[(int)i]);

    if (item != nullptr)
      listItem.push_back(item);
  }

  if (!listItem.isEmpty())
    addItems(listItem);
}

void te::qt::widgets::DoubleListWidget::removeItems(QList<QListWidgetItem*> list)
{
  std::vector<int> itemList;

  for (int i = 0; i < list.size(); ++i)
  {
    int row = m_ui->m_rightListWidget->row(list[i]);

    QListWidgetItem* item = m_ui->m_rightListWidget->takeItem(row);

    m_ui->m_leftListWidget->addItem(item);

    QVariant v = item->data(Qt::UserRole);

    if (v.isValid())
      itemList.push_back(v.toInt());
  }
  
  m_ui->m_removeToolButton->setEnabled(false);
  m_ui->m_removeAllToolButton->setEnabled(m_ui->m_rightListWidget->count() != 0);
  m_ui->m_upToolButton->setEnabled(m_ui->m_rightListWidget->count() != 0);
  m_ui->m_downToolButton->setEnabled(m_ui->m_rightListWidget->count() != 0);

  emit itemRemoved(itemList);
  emit itemChanged();
}

void te::qt::widgets::DoubleListWidget::removeItems(std::vector<int> ids)
{
  QList<QListWidgetItem*> listItem;

  for (std::size_t i = 0; i < ids.size(); ++i)
  {
    QListWidgetItem* item = getItemFromId(ids[(int)i]);

    if (item != nullptr)
      listItem.push_back(item);
  }

  if (!listItem.isEmpty())
    removeItems(listItem);
}

void te::qt::widgets::DoubleListWidget::onAddToolButtonPressed()
{
  QList<QListWidgetItem*> list = m_ui->m_leftListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_removeAllToolButton->setEnabled(true);

  addItems(list);
}

void te::qt::widgets::DoubleListWidget::onAddAllToolButtonPressed()
{
  m_ui->m_leftListWidget->selectAll();

  QList<QListWidgetItem*> list = m_ui->m_leftListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_removeAllToolButton->setEnabled(true);

  addItems(list);
}

void te::qt::widgets::DoubleListWidget::onRemoveToolButtonPressed()
{
  QList<QListWidgetItem*> list = m_ui->m_rightListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);

  removeItems(list);
}

void te::qt::widgets::DoubleListWidget::onRemoveAllToolButtonPressed()
{
  m_ui->m_rightListWidget->selectAll();

  QList<QListWidgetItem*> list = m_ui->m_rightListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);

  removeItems(list);
}

void te::qt::widgets::DoubleListWidget::onDownToolButtonPressed()
{
  int count = m_ui->m_rightListWidget->count();

  if(count == m_ui->m_rightListWidget->selectedItems().size())
    return;

  for(int i = count-1; i >= 0; --i)
  {
    QListWidgetItem* currentItem = m_ui->m_rightListWidget->item(i);

    if(i == count-1)
      continue;

    if(currentItem->isSelected())
    {
      QListWidgetItem* itemAfter = m_ui->m_rightListWidget->item(i+1);

      bool wasSelected = itemAfter->isSelected();

      m_ui->m_rightListWidget->takeItem(i);
      m_ui->m_rightListWidget->takeItem(i);

      m_ui->m_rightListWidget->insertItem(i, itemAfter);
      m_ui->m_rightListWidget->insertItem(i+1, currentItem);

      currentItem->setSelected(true);

      if(wasSelected)
        itemAfter->setSelected(true);
    }
  }
}

void te::qt::widgets::DoubleListWidget::onUpToolButtonPressed()
{
  int count = m_ui->m_rightListWidget->count();

  if(count == m_ui->m_rightListWidget->selectedItems().size())
    return;

  for(int i = 0; i < count; ++i)
  {
    QListWidgetItem* currentItem = m_ui->m_rightListWidget->item(i);

    if(i == 0)
      continue;

    if(currentItem->isSelected())
    {
      QListWidgetItem* itemBefore = m_ui->m_rightListWidget->item(i-1);

      bool wasSelected = itemBefore->isSelected();

      m_ui->m_rightListWidget->takeItem(i);
      m_ui->m_rightListWidget->takeItem(i-1);

      m_ui->m_rightListWidget->insertItem(i-1, currentItem);
      m_ui->m_rightListWidget->insertItem(i, itemBefore);

      currentItem->setSelected(true);

      if(wasSelected)
        itemBefore->setSelected(true);
    }
  }
}

void te::qt::widgets::DoubleListWidget::onLeftListItemSelectionChanged()
{
  if(m_ui->m_leftListWidget->selectedItems().size() > 0)
  {
    m_ui->m_addToolButton->setEnabled(true);
    m_ui->m_addAllToolButton->setEnabled(true);
  }
  else
  {
    m_ui->m_addToolButton->setEnabled(false);
  }
}

void te::qt::widgets::DoubleListWidget::onRightListItemSelectionChanged()
{
  if(m_ui->m_rightListWidget->selectedItems().size() > 0)
  {
    m_ui->m_removeToolButton->setEnabled(true);
    m_ui->m_removeAllToolButton->setEnabled(true);
    m_ui->m_upToolButton->setEnabled(true);
    m_ui->m_downToolButton->setEnabled(true);
  }
  else
  {
    m_ui->m_removeToolButton->setEnabled(false);
    m_ui->m_upToolButton->setEnabled(false);
    m_ui->m_downToolButton->setEnabled(false);
  }
}



QListWidgetItem* te::qt::widgets::DoubleListWidget::getItemFromId(const int &id)
{
  int count = m_ui->m_leftListWidget->count();

  for (int i = 0; i < count; ++i)
  {
    QVariant v = m_ui->m_leftListWidget->item(i)->data(Qt::UserRole);
    if (!v.isValid())
      continue;

    if (v.toInt() == id)
      return m_ui->m_leftListWidget->item(i);
  }

  count = m_ui->m_rightListWidget->count();

  for (int i = 0; i < count; ++i)
  {
    QVariant v = m_ui->m_rightListWidget->item(i)->data(Qt::UserRole);
    if (!v.isValid())
      continue;

    if (v.toInt() == id)
      return m_ui->m_rightListWidget->item(i);
  }

  return nullptr;
}
